'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.save = exports.get = exports.list = undefined;

var _v = require('uuid/v4');

var _v2 = _interopRequireDefault(_v);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const debug = (0, _debug2.default)('data-access:item');

class ItemError extends Error {

  constructor(options) {
    super(options.message);
    this.code = options.code;
    this.message = options.message;
  }
}

const list = exports.list = db => store => (() => {
  var _ref = _asyncToGenerator(function* (query) {
    try {
      const storeCol = db.collection(store);

      const items = yield storeCol.find(query).toArray();
      return items;
    } catch (e) {
      debug(e);
      throw e;
    }
  });

  return function (_x) {
    return _ref.apply(this, arguments);
  };
})();

const get = exports.get = db => store => (() => {
  var _ref2 = _asyncToGenerator(function* (query) {
    try {
      const storeCol = db.collection(store);

      const item = yield storeCol.findOne(query);
      if (!item) {
        throw new ItemError({ message: 'ItemNotFound', code: 404 });
      }
      return item;
    } catch (e) {
      debug(e);
      throw e;
    }
  });

  return function (_x2) {
    return _ref2.apply(this, arguments);
  };
})();

const save = exports.save = db => (() => {
  var _ref3 = _asyncToGenerator(function* (item) {
    const { store, itemId } = item;

    try {
      const storeCol = db.collection(store);
      const date = new Date();

      // Add the date to the offer
      const offers = item.offers.map(function (offer) {
        return Object.assign({}, offer, { date });
      });

      const [lowestNewOffer] = offers.sort(function (x, y) {
        return x.price - y.price;
      });

      const { value } = yield storeCol.findOneAndUpdate({ itemId }, {
        $set: Object.assign({}, item, { offers, updatedAt: date }),
        $setOnInsert: { offerHistory: [lowestNewOffer], createdAt: date, groupId: item.upc || (0, _v2.default)() }
      }, { upsert: true, returnOriginal: false });

      const { offerHistory = [] } = value;
      const [lastOfferHistory] = offerHistory.sort(function (x, y) {
        return y.date - x.date;
      });

      if (offerHistory.length === 0 || lastOfferHistory.price !== lowestNewOffer.price) {
        const { value: newValue } = yield storeCol.findOneAndUpdate({ itemId }, {
          $push: { offerHistory: lowestNewOffer }
        }, { returnOriginal: false });

        return newValue;
      }

      return value;
    } catch (e) {
      throw e;
    }
  });

  return function (_x3) {
    return _ref3.apply(this, arguments);
  };
})();