import { MongoClient } from 'mongodb'
import {
  save as saveItem,
  get as getItem,
  list as listItems
} from './lib/Item'

import d from 'debug'

const debug = d('data-access:main')

const {
  DB_DATABASE: dbName,
  DB_HOST: host,
  DB_PORT: port,
  DB_POOL_SIZE: poolSize
} = process.env

const url = `mongodb://${host}:${port}`

let client

export const connect = async () => {
  try {
    if (!client) {
      debug(`Connecting to ${url}`)
      client = await MongoClient.connect(url, { poolSize })
      debug('Connected')
    }
  } catch (e) {
    console.error(e)
  }

  const db = client.db(dbName)

  return {
    Item: {
      save: saveItem(db),
      get: getItem(db),
      list: listItems(db)
    }
  }
}
