// @flow
export type SupportedStores = 'amazon' | 'bestbuy' | 'walmart' | 'target' | 'gamestop'

export type Category = {
  id: string,
  name: string
}

export type ShippingInfo = {
  price: ?string
}

export type Offer = {
  condition: ?string,
  seller: ?string,
  price: number,
  available: boolean,
  shippingInfo: ShippingInfo,
  marketplace: boolean
}

type History = { date: Date }
type OfferHistory = Offer & History

export type RakutenItem = {
  itemId: string,
  upc: ?string,
  store: SupportedStores,
  title: string,
  manufacturer: ?string,
  url: string,
  msrp: ?number,
  offers: Array<Offer>,
  thumbImage: ?string,
  mdImage: ?string,
  lgImage: ?string,
  category: Category
}

export type Item = {
  _id: string,
  itemId: string,
  groupId: string,
  upc: ?string,
  store: SupportedStores,
  title: string,
  manufacturer: ?string,
  url: string,
  msrp: ?number,
  offers: Array<Offer>,
  offerHistory: Array<OfferHistory>,
  thumbImage: ?string,
  mdImage: ?string,
  lgImage: ?string,
  category: ?Category,
  updatedAt: Date,
  createdAt: Date
}

export type ItemQuery = {
  _id?: string,
  itemId?: string,
  upc?: string,
  groupId?: string,
  category?: Category
}
