const { describe, it, after, before } = require('mocha')
const { expect } = require('chai')
const {
  save: saveUser,
  get: getUser,
  list: listUsers
} = require('../../src/lib/User')

const users = require('./users.json')

describe('User', function () {
  const {
    DB_DATABASE: dbName,
    DB_HOST: host,
    DB_PORT: port
  } = process.env

  let client
  let db

  before('Connect', function () {
    const { MongoClient } = require('mongodb')
    const url = `mongodb://${host}:${port}`

    return MongoClient.connect(url, { poolSize: 10 })
      .then(_client => {
        client = _client
        db = client.db(dbName)
      })
  })

  after('Disconnect', function () {
    return db
      .dropDatabase(dbName)
      .catch(e => e)
      .then(() => {
        client.close()
      })
  })

  describe('Save', function () {
    it('should save a user', async function () {
      const [user] = users
      const _user = await saveUser(db)(user)

      expect(_user._id).to.be.ok
      expect(_user.id).to.equal(user.id)
      expect(_user.token).to.equal(user.ok)
    })
  })

  describe('Get', function () {
    it('should retrieve a user', async function () {
      const [user] = users
      await saveUser(db)(user)

      const _user = await getUser(db)(user.id)

      expect(_user._id).to.be.ok
      expect(_user.id).to.equal(user.id)
      expect(_user.token).to.equal(user.ok)
    })

    it('should throw an exception if user id is not found', function () {
      return getUser(db)('wrongId')
        .catch(e => {
          expect(e.message).to.equal('User Not Found')
        })
    })
  })

  describe('List', function () {
    it('should retrieve a list of users', async function () {
      const [user1, user2] = users
      await saveUser(db)(user1)
      await saveUser(db)(user2)

      const [_user1, _user2] = await listUsers(db)()

      expect(_user1.id).to.equal(user1.id)
      expect(_user1.token).to.equal(user1.ok)

      expect(_user2.id).to.equal(user2.id)
      expect(_user2.token).to.equal(user2.ok)
    })
  })
})
